package bin;

import kernel.ps.ProcessHandle;
import kernel.ps.IProcess;
import lib.Color;
import lib.MathI;
import kernel.log.Log;
import kernel.ui.WindowContext;

using tink.CoreApi;

@:build(macros.Binstore.includeBin("Log", ["log"]))
class KernelLog implements IProcess {
	private var handle:ProcessHandle;
	private var ctx:WindowContext;

	public function new() {}

	public function run(handle:ProcessHandle):Void {
		this.handle = handle;

		var statelessCtx = handle.createStatelessWindowContext();
		this.ctx = statelessCtx.ctx;

		statelessCtx.setRenderFunc(this.render);

		Log.onLog.handle(() -> {
			statelessCtx.requestRender();
		});
	}

	private function render() {
		ctx.clear();
		ctx.setCursorPos(0, 0);

		var lines = Log.getLines();
		var height = ctx.getSize().y;
		var start = MathI.max(lines.length - height, 0);

		for (i in start...lines.length) {
			var line = lines[i];

			switch (line.level) {
				case Info:
					ctx.setTextColor(Color.White);
					ctx.write("[INFO] ");
				case Warn:
					ctx.setTextColor(Color.Yellow);
					ctx.write("[WARN] ");
				case Error:
					ctx.setTextColor(Color.Red);
					ctx.write("[ERRO] ");
				case Debug:
					ctx.setTextColor(Color.Gray);
					ctx.write("[DEBG] ");
				case Silly:
					ctx.setTextColor(Color.Gray);
					ctx.write("[SILL] ");
			}

			ctx.write(line.message);
			ctx.setCursorPos(0, ctx.getCursorPos().y + 1);
		}
	}
}
