package kernel.log;

import haxe.ds.ReadOnlyArray;
import haxe.macro.Context;
#if (webconsole && !macro)
import lib.Debug;
#end

using tink.CoreApi;

/**
	Central logging system.
**/
class Log {
	private static inline final MAX_LINES:Int = 100;

	public static var onLog(default, null):Signal<LogLine>;

	private static final onLogTrigger:SignalTrigger<LogLine> = new SignalTrigger();
	private static final logLines:Array<LogLine> = [];

	@:allow(kernel.Init)
	private static function init() {
		onLog = onLogTrigger.asSignal();

		#if debug
		haxe.Log.trace = function(v:Dynamic, ?infos:haxe.PosInfos) {
			Log._debug(v, infos.className);
		}
		#end
	}

	public static function _info(msg:Dynamic, ?origin:String) {
		log({
			level: Info,
			message: Std.string(msg),
			time: 0,
			origin: origin
		});
	}

	public static function _warn(msg:Dynamic, ?origin:String) {
		log({
			level: Warn,
			message: Std.string(msg),
			time: 0,
			origin: origin
		});
	}

	public static function _error(msg:Dynamic, ?origin:String) {
		log({
			level: Error,
			message: Std.string(msg),
			time: 0,
			origin: origin
		});
	}

	public static function _debug(msg:Dynamic, ?origin:String) {
		#if debug
		log({
			level: Debug,
			message: Std.string(msg),
			time: 0,
			origin: origin
		});
		#end
	}

	public static function _silly(msg:Dynamic, ?origin:String) {
		log({
			level: Silly,
			message: Std.string(msg),
			time: 0,
			origin: origin
		});
	}

	private static function log(line:LogLine) {
		logLines.push(line);

		if (logLines.length > MAX_LINES) {
			logLines.shift();
		}
		onLogTrigger.trigger(line);

		#if (webconsole && !macro)
		Debug.logToWebconsole(line);
		#end
	}

	public static function getLines():ReadOnlyArray<LogLine> {
		return logLines;
	}

	#if macro
	private static function getOrigin():String {
		var localClass = Context.getLocalClass().get();
		return localClass.name;
	}
	#end

	public macro static function info(msg:ExprOf<Dynamic>) {
		return macro {
			Log._info(${msg}, $v{getOrigin()});
		}
	}

	public macro static function warn(msg:ExprOf<Dynamic>) {
		return macro {
			Log._warn(${msg}, $v{getOrigin()});
		}
	}

	public macro static function error(msg:ExprOf<Dynamic>) {
		return macro {
			Log._error(${msg}, $v{getOrigin()});
		}
	}

	public macro static function debug(msg:ExprOf<Dynamic>) {
		return macro {
			Log._debug(${msg}, $v{getOrigin()});
		}
	}

	public macro static function silly(msg:ExprOf<Dynamic>) {
		return macro {
			Log._silly(${msg}, $v{getOrigin()});
		}
	}
}
