package lib.turtle;

import lib.turtle.Structure.Layer;
import kernel.turtle.Turtle;

class Printer {
	var structure:Structure;

	public function new(s:Structure) {
		structure = s;
	}

	private function printRow(layer:Layer, x:Int, inverse:Bool) {
		for (y in 0...layer.height()) {
			var trueY = inverse ? ((layer.height() - 1) - y) : y;
			var place = layer.get(x, trueY);

			if (place) {
				Turtle.place(Down);
			}

			if (y != layer.height() - 1) {
				Turtle.forward();
			}
		}
	}

	private function printLayer(layer:Layer, inverse:Bool) {
		for (x in 0...layer.width()) {
			var trueX = inverse ? ((layer.width() - 1) - x) : x;
			printRow(layer, trueX, inverse);

			if (x == layer.width() - 1) {
				// Don't turn on the last row
				continue;
			}

			// Turn left or right
			if (x % 2 == (inverse ? 1 : 0)) {
				Turtle.turnLeft();
				Turtle.forward();
				Turtle.turnLeft();
			} else {
				Turtle.turnRight();
				Turtle.forward();
				Turtle.turnRight();
			}
		}
	}

	public function execute() {
		Turtle.up(); // We place the block below us so move up.

		for (i in 0...structure.height()) {
			var layer = structure.getLayer(i);
			var inverse = i % 2 == 0;

			printLayer(layer, inverse);

			// Don't go up or turn around on the last layer
			if (i == structure.height() - 1) {
				continue;
			}

			Turtle.up();
			Turtle.turnLeft();
			Turtle.turnLeft();
		}
	}
}
