package lib.turtle.planner;

import kernel.turtle.TurtleMutex;
import kernel.ps.ProcessHandle;

using tink.CoreApi;

/**
	A plan repesents a pice of a squence of turtle commands.
	It acts like node in a linked list.
	It also has no interal state and can be reused.
	Once a plan has been setup it (should be) is immutable.
**/
class Plan {
	private var prev:Null<Plan>;
	private var delegate:(Executer) -> Void;

	public static function newPlan():Plan {
		return new Plan();
	}

	private function new() {}

	/**
		Do a sequence of actions
	**/
	public function act(actions:Array<Action>):Plan {
		#if debug
		if (this.delegate != null) {
			throw new Error("Can't reuse plans");
		}
		#end
		this.delegate = (ex) -> {
			trace('Act: $actions');
			for (a in actions) {
				ex.execute(a);
			}
		};

		var next = new Plan();
		next.prev = this;
		return next;
	}

	/**
		Do a sequence of actions multiple times.
	**/
	public function repeat(actions:Array<Action>, times:Int):Plan {
		#if debug
		if (this.delegate != null) {
			throw new Error("Can't reuse plans");
		}
		#end
		this.delegate = (ex) -> {
			trace('Rep: $actions');
			for (i in 0...times) {
				for (a in actions) {
					ex.execute(a);
				}
			}
		};

		var next = new Plan();
		next.prev = this;
		return next;
	}

	/**
		Append a plan to the current chain.
	**/
	public function subplan(plan:Plan):Plan {
		#if debug
		if (this.delegate != null) {
			throw new Error("Can't reuse plans");
		}
		#end
		this.delegate = (ex) -> {
			trace('Subplan');
			plan.execute(ex);
		};

		var next = new Plan();
		next.prev = this;
		return next;
	}

	/**
		Execute a plan mutiple times. 
	**/
	public function repateSubplan(plan:Plan, times:Int):Plan {
		#if debug
		if (this.delegate != null) {
			throw new Error("Can't reuse plans");
		}
		#end
		this.delegate = (ex) -> {
			trace('Subplan repeat');
			for (_ in 0...times) {
				plan.execute(ex);
			}
		};

		var next = new Plan();
		next.prev = this;
		return next;
	}

	private function execute(ex:Executer) {
		if (this.prev == null) {
			this.delegate(ex);
			return;
		}

		this.prev.execute(ex);

		if (this.delegate != null) { // The last plan in a line does not have a delegate
			this.delegate(ex);
		}

		return;
	}

	public function begin(handle:ProcessHandle) {
		trace("begin plan");
		if (!handle.claimTurtleMutex()) {
			handle.writeLine("Failed to claim turtle mutex");
			handle.close();
			return;
		}

		var ex = new Executer();

		TurtleMutex.runInTThread(() -> {
			try {
				this.execute(ex);
			} catch (e) {
				handle.writeLine(e.message);
			}
			handle.close(true);
		});
	}
}
